#include "Timer_UART.h"
#include "msp430.h"

/************ Software UART Global Variables *******************************/
unsigned int  TXData;
unsigned char BitCnt;
unsigned char UART_Busy = 0;
unsigned char Data_Trigger = 0;

void TX_Byte (void)                                                             // Function Transmits Character from TXData Buffer
{
    BitCnt = 0xA;                                                               // Load Bit counter, 8data + ST/SP
    CCR0 = TAR;                                                                 // Current state of TA counter
    CCR0 += Bitime;                                                             // Some time till first bit
    TXData |= 0x100;                                                         	// Add mark stop bit to TXData
    TXData = TXData << 1;                                                   	// Add space start bit
    CCTL0 =  CCIS0 + OUTMOD0 + CCIE;                                            // TXD = mark = idle
    while ( CCTL0 & CCIE );                                                     // Wait for TX completion
}

void TimerA_UART_tx(unsigned char byte)
{
    
    BitCnt = 0xA;						                 						// Ensure last char got TX'd
    TA0CCR0 = TAR;                           									// Current state of TA counter
    TA0CCR0 += Bitime;                    										// One bit time till first bit
    
    TXData = byte;                          									// Load global variable
    TXData |= 0x100;                        									// Add mark stop bit to TXData
    TXData <<= 1;                           									// Add space start bit
    
    TA0CCTL0 = CCIS0 + OUTMOD0 + CCIE;               							// Set TXD on EQU0, Int
    while (TA0CCTL0 & CCIE);
}


void Print_f(unsigned char* string, unsigned char size)
{
    unsigned char i;

    if (!UART_Busy)                                                             // Is the UART Busy Receiving a byte?
    {
      for (i=0; i< size; i++)
      {
           TXData = string[i];                                                // Load the data into the TX Buffer
           TX_Byte();                                                           // And transmit
      }
    }
}

/**************************************************************************************************************************************************
*                                 UART Hex Conversion and New Line Carriage Return Utilities                                                      *
**************************************************************************************************************************************************/
void Print_Hex_Word (unsigned int Data, unsigned char type)
{
    unsigned char Hex_Data [6];
    unsigned char Data_Place = 5;
    signed char l=0;
    if (type == 24)
    {
    	unsigned char bin_Data[8];
    		
    	for(l=7; l>=0; l--)
    	{
    		bin_Data[l] = '0' + (Data%2);
    		Data=Data-(Data%2);
    		Data=Data/2;
    		
    	}
    	Print_f(bin_Data,8);
    		
    }
    else
    {

    if (type == VOLTS)
    {
      Data_Place--;
    }

    Hex_Data[Data_Place] = '0'+(Data%10);                                       //Bit 5 for both TEMP and VOLTS
    Data_Place--;

    if (type == TEMP)                                                           // Bit 4 if TEMP
    {
      Hex_Data[Data_Place] = '.';
      Data_Place--;
    }

    Hex_Data[Data_Place] = '0'+(Data/10)%10;                                    // Bit 3 if TEMP; bit 4 if VOLTS
    Data_Place--;

    Hex_Data[Data_Place] = '0'+(Data/100)%10;                                   // Bit 2 if TEMP; bit 3 if VOLTS
    Data_Place--;

    if (type == VOLTS)
    {
      Hex_Data[Data_Place] = '.';                                               // Bit 1 if VOLTS
      Data_Place--;
    }

    Hex_Data[Data_Place] = '0'+(Data/1000)%10;                                  //Bit 1 if TEMP; bit 2 if VOLTS
    Data_Place--;

    Hex_Data[Data_Place] = '0'+(Data/10000)%10;                                 // Bit 0 for both TEMP and VOLTS
    Data_Place--;

    if (type == VOLTS)
    {
        Print_f(Hex_Data,5);
    }
    else
        Print_f(Hex_Data,6);  
    }                                                  // Print the Temp Value                                                       // Print the Temp Value
}

/*
 *  Timer0_A3 ISR. Enable Timer0_A3 CCR0 interrupt and set Timer_ISR as the interrupt handler.
 *  Then set the timer interrupt to Manual on interrupt return.
 */
void Timer_TX_ISR (void)
{
    CCR0 += Bitime;                                                              // Add Offset to CCR0
    if (CCTL0 & CCIS0)                                                           // TX on CCI0B?
    {
        if ( BitCnt == 0)
        {
            CCTL0 &= ~ CCIE;                                                     // All bits TXed, disable interrupt
        }
        else
        {
            CCTL0 |=  OUTMOD2;                                                   // TX Space
            if (TXData & 0x01)
            {
                CCTL0 &= ~ OUTMOD2;                                              // TX Mark
            }
            TXData = TXData >> 1;
            BitCnt --;
        }
    }
}

