#include "msp430g2553.h"
#include "BoosterPack.h"
#include "Single_wire.h"

void init_TPL0401_Booster(void);
void WDT_init(void);
void GPIO_init(void);
void CLK_init(void);
void I2C_init(void);
void Timers_init(void);
void System_init(void);
void decode_Button(void);
//extern void TCA5405_init(void);

void init_TPL0401_Booster (void)
{
	WDT_init();                         //Initialize the WatchDog Timer
	GPIO_init();						//Initialize GPIO
	CLK_init();							//Initialize SMCLK DOCLK	
	I2C_init();							//Initialize I2C Bus
	Timers_init();						//Initialize Timer A0 for serial communication
	TCA5405_init();						//Initialize Timer A1 for TCA5405 communication
	System_init();						//Initialize System interrupts for I2C Bus 
}

void WDT_init(void)
{
	 WDTCTL = WDTPW + WDTTMSEL + WDTIS0;						// Enable WDT SMCLK / 8192
    //WDTCTL = WDTPW + WDTTMSEL + WDTSSEL + WDTIS1;
    
}

void GPIO_init(void)
{
	P1OUT = DIN;											    //Set BIT4 HIGH for Single wire communication
    P1SEL = BIT1 +BIT2 + BIT6 + BIT7;							//Select bits for UART
    P1SEL2 = BIT6 + BIT7;										//and I2C
    P1DIR = BIT0 + BIT1;//+ DIN;									//Set P1.0 and P1.1 to outputs all other to input
    P1IES = BIT5;												//Flag an interrupt on a HIGH to LOW transition

    P1IFG &= ~BIT5;													//Clear interrupt flag
    P1IE = BIT5;												//Enable interrupt on P1.3;
	
	//The following P2 setup is required for TimerA1 UART
	//for TCA5405 communication
	P2OUT = BIT0;
	P2SEL = BIT0;
    P2DIR = BIT0;

}

void CLK_init(void)
{
	BCSCTL2 = SELM_0 + DIVM_0 + DIVS_0;

    if (CALBC1_8MHZ != 0xFF) {
        											/* Adjust this accordingly to your VCC rise time */
        __delay_cycles(100000);
        											// Follow recommended flow. First, clear all DCOx and MODx bits. Then
        											// apply new RSELx values. Finally, apply new DCOx and MODx bit values.
        DCOCTL = 0x00;
        BCSCTL1 = CALBC1_8MHZ;      				/* Set DCO to 8MHz */
        DCOCTL = CALDCO_8MHZ;
    }
    BCSCTL1 |= XT2OFF + DIVA_0;
    BCSCTL3 = XT2S_0 + LFXT1S_2 + XCAP_1;
    	
}

void I2C_init(void)
{
	UCB0CTL1 |= UCSWRST;							//Set UCSWRST to configure I2C bus
    UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;			//I2C, synchronous master mode
    UCB0CTL1 = UCSSEL_2 + UCSWRST;					//Use 8MHz SMCLK 
    UCB0BR0 = 20;									//Set Bitrate register 8MHz/20 = 400kHz
    UCB0CTL1 &= ~UCSWRST;							//Reset UCSWRST to enable I2C communicatio
    
    	
}

void Timers_init(void)
{
    TA0CCTL0 = CM_0 + CCIS_0 + OUTMOD_0 + OUT;				//CCR0 compare mode, outmode 0, interrupt enable (UART TX)												
    TACCTL1 = SCS + CM1 + CAP + CCIE;						//CCR1 sync, neg edge, capture, interrupt enable (UART RX)
    TA0CTL = TASSEL_2 + ID_3 + MC_2;						//SMCLK Div 8 continuous mode
}

void System_init(void)
{
    IFG2 &= ~(UCB0TXIFG + UCB0RXIFG);						//Clear USCI flags
    IE2 |= UCB0TXIE + UCB0RXIE;								//Enable USCI interrupts for I2C 	
}
